/*
 * dfd_set.h copyright (c) 1996 Clemson University, all rights reserved.
 *
 * Written by Rob Ross, updated by Matt Cettei.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Contact:  Rob Ross    rbross@parl.eng.clemson.edu
 */

#ifndef __DYN_FDSET_H__
#define __DYN_FDSET_H__

#include <config.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

/* struct dyn_fdset:
 * Replacement for fd_set that supports a variably-sized region for
 * storing the bits that determine what fds are selected.
 * 
 * Fields:
 * magic - set to DFD_MAGIC on allocation, 0 on free, to help with
 *   debugging
 * alloc_size - allocated size, in bytes
 * set - pointer to data region storing bits
 *
 * Semantics:
 * - if we haven't allocated space for a FD, then it is not set.
 * - if user sets a bit we haven't allocated space for, we allocate
 *    space and then set it.
 *
 * Usage:
 * - a dyn_fdset must be initialized via dfd_init() before use.
 * - a dyn_fdset must be freed with dfd_finalize() if it is no longer
 *   needed, or memory will be lost.
 * - a dyn_fdset must be cleaned up via dfd_finalize() if it is going to
 *   be re-initialized.  Note, however, that re-initialization is
 *   usually unnecessary; DFD_ZERO() should do the trick in most cases.
 */
struct dyn_fdset {
	int magic;
	int32_t alloc_size;
	void *set;
};

typedef struct dyn_fdset dyn_fdset;

/* initialization/cleanup functions */
int dfd_init(dyn_fdset *dset, int startsz);
int dfd_finalize(dyn_fdset *dset);

/* the four traditional fd_set operations */
int dfd_set(int fd, dyn_fdset *dset);
int dfd_clr(int fd, dyn_fdset *dset);
int dfd_isset(int fd, dyn_fdset *dset);
int dfd_zero(dyn_fdset *dset);

/* copy function */
int dfd_copy(dyn_fdset *dest, dyn_fdset *src);

/* we need a select that will operate on these guys too */
int dfd_select(int n, dyn_fdset *readfds, dyn_fdset *writefds, dyn_fdset
*exceptfds, struct timeval *timeout);

/* debugging functions */
int dfd_dump(dyn_fdset *dset);

/* some defines to make things look more like the old fd_set calls */
#define DFD_SET(__a,__b) dfd_set((__a),(__b))
#define DFD_CLR(__a,__b) dfd_clr((__a),(__b))
#define DFD_ISSET(__a,__b) dfd_isset((__a),(__b))
#define DFD_ZERO(__a) dfd_zero((__a))



enum {
	DFD_MAGIC = 0x4321dcba,
	DFD_MIN_ALLOCSZ = 256
};

#endif
